@@ -1,12 +1,12 @@
 package Test::BDD::Cucumber::Executor;
-$Test::BDD::Cucumber::Executor::VERSION = '0.75';
+$Test::BDD::Cucumber::Executor::VERSION = '0.84';
 =head1 NAME
 
 Test::BDD::Cucumber::Executor - Run through Feature and Harness objects
 
 =head1 VERSION
 
-version 0.75
+version 0.84
 
 =head1 DESCRIPTION
 
@@ -18,13 +18,10 @@ Definitions, and reporting on progress through the passed-in harness.
 use Moo;
 use MooX::HandlesVia;
 use Types::Standard qw( Bool Str ArrayRef HashRef );
-use Clone qw(clone);
 use List::Util qw/first any/;
-use List::MoreUtils qw/pairwise/;
 use Module::Runtime qw/use_module/;
-use Number::Range;
-use Carp qw/croak/;
-our @CARP_NOT;
+use utf8;
+use Encode ();
 
 use Test2::API qw/intercept/;
 
@@ -43,8 +40,6 @@ use Test::BDD::Cucumber::Util;
 use Test::BDD::Cucumber::Model::Result;
 use Test::BDD::Cucumber::Errors qw/parse_error_from_line/;
 
-has '_bail_out' => ( is => 'rw', isa => Bool, default => 0 );
-
 =head1 ATTRIBUTES
 
 =head2 matching
@@ -228,7 +223,6 @@ sub execute_outline {
                     scenario => $outline,
                     background => $background,
                     scenario_stash => {},
-                    outline_state => {},
                     dataset => {},
                 });
         }
@@ -244,8 +238,6 @@ sub execute_outline {
 
     foreach my $rows (@datasets) {
 
-        my $outline_state = {};
-
         foreach my $row (@{$rows->data}) {
 
             my $name = $outline->{name} || "";
@@ -261,11 +253,8 @@ sub execute_outline {
                     scenario => $outline,
                     background => $background,
                     scenario_stash => {},
-                    outline_state => $outline_state,
                     dataset => $row,
                 });
-
-            $outline_state->{'short_circuit'} ||= $self->_bail_out;
         }
     }
 }
@@ -297,10 +286,10 @@ the Harness interface.
 sub _execute_steps {
     my ( $self, $options ) = @_;
     my ( $feature, $feature_stash, $harness, $outline,
-        $scenario_stash, $outline_state, $dataset, $context_defaults )
+        $scenario_stash, $scenario_state, $dataset, $context_defaults )
       = @$options{
         qw/ feature feature_stash harness scenario scenario_stash
-          outline_state dataset context_defaults
+          scenario_state dataset context_defaults
           /
       };
 
@@ -335,11 +324,11 @@ sub _execute_steps {
 
         my $result =
             $self->find_and_dispatch( $context,
-                                      $outline_state->{'short_circuit'}, 0 );
+                                      $scenario_state->{'short_circuit'}, 0 );
 
         # If it didn't pass, short-circuit the rest
         unless ( $result->result eq 'passing' ) {
-            $outline_state->{'short_circuit'}++;
+            $scenario_state->{'short_circuit'}++;
         }
 
     }
@@ -349,7 +338,7 @@ sub _execute_steps {
 
 
 sub _execute_hook_steps {
-    my ( $self, $phase, $context_defaults, $outline_state ) = @_;
+    my ( $self, $phase, $context_defaults, $scenario_state ) = @_;
     my $want_short = ($phase eq 'before');
 
     for my $step ( @{ $self->{'steps'}->{$phase} || [] } ) {
@@ -360,13 +349,13 @@ sub _execute_hook_steps {
         my $result =
             $self->dispatch(
                 $context, $step,
-                ($want_short ? $outline_state->{'short_circuit'} : 0),
+                ($want_short ? $scenario_state->{'short_circuit'} : 0),
                 0 );
 
         # If it didn't pass, short-circuit the rest
         unless ( $result->result eq 'passing' ) {
             if ($want_short) {
-                $outline_state->{'short_circuit'} = 1;
+                $scenario_state->{'short_circuit'} = 1;
             }
         }
     }
@@ -378,12 +367,13 @@ sub _execute_hook_steps {
 sub execute_scenario {
     my ( $self, $options ) = @_;
     my ( $feature, $feature_stash, $harness, $outline, $background_obj,
-        $scenario_stash, $outline_state, $dataset )
+        $scenario_stash, $dataset )
       = @$options{
         qw/ feature feature_stash harness scenario background scenario_stash
-          outline_state dataset
+          dataset
           /
-      };
+    };
+    my $scenario_state = {};
 
     my %context_defaults = (
         executor => $self,    # Held weakly by StepContext
@@ -412,7 +402,7 @@ sub execute_scenario {
     $_->pre_scenario( $outline, $feature_stash, $scenario_stash )
         for @{ $self->extensions };
 
-    $self->_execute_hook_steps( 'before', \%context_defaults, $outline_state );
+    $self->_execute_hook_steps( 'before', \%context_defaults, $scenario_state );
 
     if ($background_obj) {
         $harness->background( $outline, $dataset,
@@ -424,7 +414,7 @@ sub execute_scenario {
                 feature_stash  => $feature_stash,
                 harness        => $harness,
                 scenario_stash => $scenario_stash,
-                outline_state  => $outline_state,
+                scenario_state  => $scenario_state,
                 context_defaults => \%context_defaults,
             }
             );
@@ -438,15 +428,15 @@ sub execute_scenario {
             feature_stash  => $feature_stash,
             harness        => $harness,
             scenario_stash => $scenario_stash,
-            outline_state  => $outline_state,
+            scenario_state  => $scenario_state,
             dataset        => $dataset,
             context_defaults => \%context_defaults,
         });
 
-    $self->_execute_hook_steps( 'after', \%context_defaults, $outline_state );
+    $self->_execute_hook_steps( 'after', \%context_defaults, $scenario_state );
 
     $_->post_scenario( $outline, $feature_stash, $scenario_stash,
-                       $outline_state->{'short_circuit'} )
+                       $scenario_state->{'short_circuit'} )
         for reverse @{ $self->extensions };
 
     $harness->scenario_done( $outline, $dataset );
@@ -594,9 +584,7 @@ sub dispatch {
     # Say we're about to start it up
     $context->harness->$step_name($context);
 
-    # Store the string position of matches for highlighting
     my @match_locations;
-
     my $stash_keys = join ';', sort keys %{$context->stash};
     # Using `intercept()`, run the step function in an isolated
     # environment -- this should not affect the enclosing scope
@@ -611,9 +599,6 @@ sub dispatch {
         ###TODO: Both intercept() and Test::More::subtest() should
         # be replaced by a specific Hub implementation for T::B::C
         Test::More::subtest( 'execute step', sub {
-            pass(
-                "Starting to execute step: " . $context->text
-                );
 
             # Take a copy of this. Turns out actually matching against it
             # directly causes all sorts of weird-ass heisenbugs which mst has
@@ -626,9 +611,12 @@ sub dispatch {
             # Save the location of matched subgroups for highlighting hijinks
             my @starts = @-;
             my @ends   = @+;
-            @match_locations = pairwise { [ $a, $b ] } @starts, @ends;
+
+            # Store the string position of matches for highlighting
+            @match_locations = map { [ $_, shift @ends ] } @starts;
 
             # OK, actually execute
+            local $@;
             eval {
                 no warnings 'redefine';
 
@@ -642,7 +630,10 @@ sub dispatch {
                 $coderef->($context)
             };
             if ($@) {
-                fail("Step ran successfully", $@);
+                fail("Step ran to completion", "Exception: ", $@);
+            }
+            else {
+                pass("Step ran to completion");
             }
 
             done_testing();
@@ -659,7 +650,7 @@ sub dispatch {
             # from out of the other results first.
             output => $self->_test_output(
                 (first { $_->isa('Test2::Event::Subtest') }
-                 @$events)->{subevents})
+                 @$events)->{subevents}),
         });
     warn qq|Unsupported: Step modified C->stash instead of C->stash->{scenario} or C->stash->{feature}|
         if $stash_keys ne (join ';', sort keys %{$context->stash});
@@ -690,21 +681,13 @@ sub _extract_match_strings {
 
     return unless @match_locations;
 
-    # Consolidate overlaps
-    my $range = Number::Range->new();
-
-    {
-        # Don't want a complain about numbers already in range, as that's
-        # expected for nested matches
-        no warnings;
-        $range->addrange( $_->[0] . '..' . ( $_->[1] - 1 ) )
-          for @match_locations;
-    }
+    my %range =
+        map { $_ => 1 } map { $_->[0] .. ($_->[1] - 1) } @match_locations;
 
     # Walk the string, splitting
     my @parts = ( [ 0, '' ] );
     for ( 0 .. ( ( length $text ) - 1 ) ) {
-        my $to_highlight = $range->inrange($_);
+        my $to_highlight = $range{$_} || 0;
         my $character = substr( $text, $_, 1 );
 
         if ( $parts[-1]->[0] != $to_highlight ) {
@@ -720,14 +703,14 @@ sub _extract_match_strings {
 sub _test_output {
     my ($self, $events) = @_;
     my $fmt = Test2::Formatter::TAP->new();
-    open my $stdout, '>', \my $out_text;
+    open my $stdout, '>:encoding(UTF-8)', \my $out_text;
     my $idx = 0;
 
     $fmt->set_handles([ $stdout, $stdout ]);
     $self->_test_output_from_subevents($events, $fmt, \$idx);
     close $stdout;
 
-    return $out_text;
+    return Encode::decode('utf8', $out_text);
 }
 
 sub _test_output_from_subevents {
@@ -813,7 +796,7 @@ Peter Sergeant C<pete@clueball.com>
 
 =head1 LICENSE
 
-  Copyright 2019-2020, Erik Huelsmann
+  Copyright 2019-2023, Erik Huelsmann
   Copyright 2011-2019, Peter Sergeant; Licensed under the same terms as Perl
 
 =cut
